


System = class()

function System:init()
    -- Initialization for generic system rules
end

function System:setup(grid)
    -- Setup logic specific to the system
end

function System:applyRule(grid, row, col)
    -- To be overridden in subclasses
    print("applyRule not implemented")
end





-- Conway's Game of Life as a System object

ConwaysGameOfLife = class(System)

function ConwaysGameOfLife:init()
    -- Initialization
end

function ConwaysGameOfLife:setup(grid)
    -- Initialize the grid, if needed
    for i = 1, grid.rows do
        for j = 1, grid.cols do
            grid.cells[i][j] = math.random(0, 1)  -- Randomly assign cell as alive (1) or dead (0)
        end
    end
end

function ConwaysGameOfLife:applyRule(grid, row, col)
    local neighborCount = self:countNeighbors(grid, row, col)
    local cell = grid:getCell(row, col, true) -- Assuming wrap-around behavior
    local returnValue = (neighborCount == 3 or (cell == 1 and neighborCount == 2)) and 1 or 0
    return returnValue
end

function ConwaysGameOfLife:countNeighbors(grid, row, col)
    local count = 0
    for i = -1, 1 do
        for j = -1, 1 do
            if not (i == 0 and j == 0) then
                local neighbor = grid:getCell(row + i, col + j, true) -- Wrap-around
                count = count + (self:isAlive(neighbor) and 1 or 0)
            end
        end
    end
    return count
end

function ConwaysGameOfLife:isAlive(cell)
    return cell == 1
end






-- A self-nesting version of the Game of Life

NestingGameOfLife = class(ConwaysGameOfLife)

function NestingGameOfLife:init()
    ConwaysGameOfLife.init(self)  -- Call the superclass initializer
end

function NestingGameOfLife:setup(grid)
    self.grid = grid
    for i = 1, grid.rows do
        for j = 1, grid.cols do
            if math.random(0, 1) == 1 then
                -- Randomly decide if a cell should be a nested CA
                grid.cells[i][j] = self:createNestedCA()
            else
                grid.cells[i][j] = 0  -- Non-nested cell
            end
        end
    end
end

function NestingGameOfLife:applyRule(grid, row, col)
    local cell = grid:getCell(row, col, true)  -- Assuming wrap-around behavior
    local isAlive = self:isAlive(cell)
    local neighborCount = self:countNeighbors(grid, row, col)
    if isAlive and (neighborCount == 2 or neighborCount == 3) then
        return cell  -- Cell stays alive
    elseif not isAlive and neighborCount == 3 then
        return self:createNestedCA()  -- Cell becomes a nested CA
    else
        return 0  -- Cell dies
    end
end

function NestingGameOfLife:createNestedCA()
    local nestedCA = CellularAutomata({ConwaysGameOfLife()}, self.grid.rows, self.grid.cols)
    --nestedCA.color = self:randomColor()  -- Assign a random color
    return nestedCA
end

function NestingGameOfLife:randomColor()
    return color(math.random(255), math.random(255), math.random(255))
end

function NestingGameOfLife:isAlive(cell)
    return cell == 1 or (type(cell) == "table" and cell.class == "CellularAutomata")
end
