

Updater = class()

function Updater:init(ca)
    self.ca = ca
end

function Updater:update()
    self:updateCA(self.ca)
end

function Updater:updateCA(ca)
    -- Create a new table for the updated states
    local newStates = {}
    for i = 1, ca.grid.rows do
        newStates[i] = {}
        for j = 1, ca.grid.cols do
            local contents = ca.grid:getCell(i, j)
            if type(contents) == "table" and contents.class == "CellularAutomata" then
                -- Recursively update the nested CA
                self:updateCA(contents)
                -- Then apply the system rules at the current level as well
                newStates[i][j] = self:applySystemsRules(ca, i, j)
            else
                -- Apply the system rules for regular cells
                newStates[i][j] = self:applySystemsRules(ca, i, j)
            end
        end
    end
    
    -- Update the CA's grid cells with the new states
    for i = 1, ca.grid.rows do
        for j = 1, ca.grid.cols do
            ca.grid.cells[i][j] = newStates[i][j]
        end
    end
end

function Updater:applySystemsRules(ca, row, col)
    local finalState = ca.grid.cells[row][col]
    for _, system in ipairs(ca.systems) do
        finalState = system:applyRule(ca.grid, row, col)
    end
    return finalState
end
