

Visualizer = class()

function Visualizer:init(ca)
    self.ca = ca
    self:calculateCellSizeAndOffset()
end

function Visualizer:draw()
    for i = 1, self.ca.grid.rows do
        for j = 1, self.ca.grid.cols do
            local cell = self.ca.grid.cells[i][j]
            local x = (j - 1) * self.cellWidth + self.offsetX
            local y = (i - 1) * self.cellHeight + self.offsetY
            
            if type(cell) == "table" and cell.class == "CellularAutomata" then
                -- Draw the nested CA
                self:drawNestedCA(cell, x, y, self.cellWidth, self.cellHeight)
            else
                -- Draw regular cell (alive cells in white, dead cells in black)
                fill(cell == 1 and color(255) or color(0))
                rect(x, y, self.cellWidth, self.cellHeight)
            end
        end
    end
end

function Visualizer:drawNestedCA(nestedCA, x, y, width, height)
    local nestedCellWidth = width / nestedCA.grid.cols
    local nestedCellHeight = height / nestedCA.grid.rows
    
    -- Draw semi-transparent background if nestedCA has a color
    if nestedCA.color then
        fill(0)
        rect(x, y, width, height)
        fill(nestedCA.color.r, nestedCA.color.g, nestedCA.color.b, 61) -- 51 is about 20% opacity
        rect(x, y, width, height)
    end
    
    for i = 1, nestedCA.grid.rows do
        for j = 1, nestedCA.grid.cols do
            local cell = nestedCA.grid:getCell(i, j)
            local nestedX = x + (j - 1) * nestedCellWidth
            local nestedY = y + (i - 1) * nestedCellHeight
            
            if type(cell) == "table" and cell.class == "CellularAutomata" then
                -- Recursively draw deeper nested CA
                self:drawNestedCA(cell, nestedX, nestedY, nestedCellWidth, nestedCellHeight)
            elseif cell == 1 then
                -- Draw the nested cell
                fill(nestedCA.color or color(10, 110, 220))
                rect(nestedX, nestedY, nestedCellWidth * 1.65, nestedCellHeight * 1.65)
            end
        end
    end
end

function Visualizer:calculateCellSizeAndOffset()
    local margin = math.min(WIDTH, HEIGHT) / 23
    local maxCellsInRow = math.max(self.ca.grid.rows, self.ca.grid.cols)
    local minDimension = math.min(WIDTH, HEIGHT)
    local cellSize = (minDimension - 2 * margin) / maxCellsInRow
    
    self.cellWidth = cellSize
    self.cellHeight = cellSize
    
    -- Calculate offset to center the grid
    self.offsetX = (WIDTH - self.cellWidth * self.ca.grid.cols) / 2
    self.offsetY = (HEIGHT - self.cellHeight * self.ca.grid.rows) / 2
end
